CREATE TABLE country (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	a2 VARCHAR(2) UNIQUE,
	a3 VARCHAR(3) UNIQUE,
	ccode INT UNIQUE,
	name VARCHAR(256) UNIQUE,
	rir_id INT,
	lat  REAL,
	long REAL,
	capital_id INT,
	flag_id INT,
	FOREIGN KEY (capital_id) REFERENCES city(id),
	FOREIGN KEY (flag_id)    REFERENCES flags(id)
);

CREATE TABLE city (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	country_id INT,
	name VARCHAR(256),
	lat REAL,
	long REAL,
	UNIQUE (country_id, name, lat, long)	
);

CREATE TABLE flags (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	country_id INT,	
	icon BLOB,
	image BLOB
);

CREATE TABLE country2phoneprefix (
	country_id INT,	
	prefix INT(8),
	FOREIGN KEY (country_id) REFERENCES country(id)
);

CREATE TABLE inet6num (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	inetinfo_id INTEGER,
	prefix VARCHAR(64),
	length INTEGER,
	prefix_hi INTEGER(8),
	prefix_lo INTEGER(8),
	mask_hi   INTEGER(8),
	mask_lo   INTEGER(8),
	FOREIGN KEY (inetinfo_id) REFERENCES inetinfo(id),
	UNIQUE (prefix_hi, prefix_lo, mask_hi, mask_lo)
);

CREATE TABLE inetinfo2contact_adminc (
       inetinfo_id INTEGER,
       contact_id  INTEGER, 
       FOREIGN KEY (inetinfo_id) REFERENCES inetinfo(id),
       FOREIGN KEY (contact_id)  REFERENCES contact(id),
       UNIQUE (inetinfo_id, contact_id)
);

CREATE TABLE inetinfo2contact_techc (
       inetinfo_id INTEGER,
       contact_id INTEGER,
       FOREIGN KEY (inetinfo_id) REFERENCES inetinfo(id),
       FOREIGN KEY (contact_id)  REFERENCES contact(id),
       UNIQUE (inetinfo_id, contact_id)
);

CREATE TABLE inetinfo (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	netname      VARCHAR(256),
	country_id   INTEGER,
	city_id      INTEGER,
	status_id    INTEGER,
	billingc_id  INTEGER,
	desc         VARCHAR(4096),
	registrar_id INTEGER,
	mnt_domain   VARCHAR(256),
	mnt_routes   VARCHAR(256),
	rev_srv      VARCHAR(256),
	notify_id    INTEGER,
	remarks      VARCHAR(256),
	source_id    INTEGER,
	FOREIGN KEY (country_id)   REFERENCES country(id),
	FOREIGN KEY (city_id)      REFERENCES city(id),
	FOREIGN KEY (billingc_id)  REFERENCES contact(id),
	FOREIGN KEY (registrar_id) REFERENCES registrar(id),
	FOREIGN KEY (status_id)    REFERENCES status(id),
        FOREIGN KEY (source_id)    REFERENCES source(id)
);
	

-- Main table for DNS servers information. Used for rev-srv

CREATE TABLE dns (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	dns VARCHAR(256) UNIQUE
);

CREATE TABLE inetinfo2revsrv (
       inetinfo_id INTEGER,
       revsrv_id   INTEGER,
       FOREIGN KEY (inetinfo_id) REFERENCES inetinfo(id),
       FOREIGN KEY (revsrv_id)   REFERENCES dns(id),
       UNIQUE (inetinfo_id, revsrv_id)
);


-- A specific table for email addresses

CREATE TABLE email (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	email VARCHAR(256) UNIQUE
);

CREATE TABLE inetinfo2email_notify (
       inetinfo_id INTEGER,
       email_id  INTEGER,
       FOREIGN KEY (inetinfo_id) REFERENCES inetinfo(id),
       FOREIGN KEY (email_id)    REFERENCES email(id),
       UNIQUE (inetinfo_id, email_id)
);


-- a table for 'changed' entry information 

CREATE TABLE changed (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	inetinfo_id INTEGER,
	date DATE,
	email_id INTEGER,
        FOREIGN KEY (email_id)    REFERENCES email(id),
	FOREIGN KEY (inetinfo_id) REFERENCES inetinfo(id)
);

CREATE TABLE registrar (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	name VARCHAR(256)
);

CREATE TABLE status (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	status VARCHAR(256),
	UNIQUE(status)
);



-- based on 'person' and 'role' object in ripe database (sect 1.2.14 
-- and 1.2.15 of ripe-252). compared to 'person', 'role' object has 
-- also trouble, adminc, techc attribute. We name that a contact.
-- 'person' and 'role' attribute are now a single attribuet called 
-- 'name'

CREATE TABLE contact (
	id         INTEGER PRIMARY KEY AUTOINCREMENT,
	name       VARCHAR(256),
	address    VARCHAR(256),
	phone      VARCHAR(32),
	fax        VARCHAR(32),
	email_id   INTEGER,
	nic_hdl    VARCHAR(32),
	trouble    VARCHAR(256), -- review me
	adminc_id  INTEGER,
	techc_id   INTEGER,
	remarks	   VARCHAR(4096),
	notify_id  INTEGER,
	mnt_by_id  INTEGER,
	changed_id INTEGER,
	source_id  INTEGER,
	UNIQUE(nic_hdl),
	UNIQUE(name),
        FOREIGN KEY (source_id) REFERENCES source(id)
);


CREATE TABLE phone (
        id    	   INTEGER PRIMARY KEY AUTOINCREMENT,
	phone 	   VARCHAR(16),
	UNIQUE (phone)
);


CREATE TABLE contact2phone (
       contact_id INTEGER,
       phone_id  INTEGER,
       FOREIGN KEY (contact_id) REFERENCES contact(id),
       FOREIGN KEY (phone_id)    REFERENCES phone(id),
       UNIQUE (contact_id, phone_id)
);

CREATE TABLE fax (
        id    	   INTEGER PRIMARY KEY AUTOINCREMENT,
	fax 	   VARCHAR(16),
	UNIQUE (fax)
);


CREATE TABLE contact2fax (
       contact_id INTEGER,
       fax_id  INTEGER,
       FOREIGN KEY (contact_id) REFERENCES contact(id),
       FOREIGN KEY (fax_id)     REFERENCES fax(id),
       UNIQUE (contact_id, fax_id)
);


-- based on 'mntner' object in ripe database (sect 1.2.12 or ripe-252)

CREATE TABLE mntner (
	id             INTEGER PRIMARY KEY AUTOINCREMENT,
	mntner         VARCHAR(256),
	descr	       VARCHAR(4096),
	adminc_id      INTEGER,	   
	techc_id       INTEGER,
	updto	       INTEGER,
	mnt_nfy_id     INTEGER,
	auth	       VARCHAR(256),
	remarks	       VARCHAR(4096),
	notify_id      INTEGER,
	mnt_by_id      INTEGER,
	referral_by_id INTEGER,
	changed        INTEGER,
	source_id      INTEGER,
        FOREIGN KEY (source_id) REFERENCES source(id),
	UNIQUE(mntner)
);

CREATE TABLE inetinfo2mntner_mntby (
       inetinfo_id    INTEGER,
       mntner_id      INTEGER,
       FOREIGN KEY (inetinfo_id) REFERENCES inetinfo(id),
       FOREIGN KEY (mntner_id)   REFERENCES mntner(id),
       UNIQUE (inetinfo_id, mntner_id)
);

CREATE TABLE inetinfo2mntner_mntroutes (
       inetinfo_id    INTEGER,
       mntner_id      INTEGER,
       FOREIGN KEY (inetinfo_id) REFERENCES inetinfo(id),
       FOREIGN KEY (mntner_id)   REFERENCES mntner(id),
       UNIQUE (inetinfo_id, mntner_id)
);

CREATE TABLE inetinfo2mntner_mntlower (
       inetinfo_id    INTEGER,
       mntner_id      INTEGER,
       FOREIGN KEY (inetinfo_id) REFERENCES inetinfo(id),
       FOREIGN KEY (mntner_id)   REFERENCES mntner(id),
       UNIQUE (inetinfo_id, mntner_id)
);

CREATE TABLE inetinfo2mntner_mntdomains (
       inetinfo_id    INTEGER,
       mntner_id      INTEGER,
       FOREIGN KEY (inetinfo_id) REFERENCES inetinfo(id),
       FOREIGN KEY (mntner_id)   REFERENCES mntner(id),
       UNIQUE (inetinfo_id, mntner_id)
);


-- IRT object. Defined in RIPE-254

CREATE TABLE irt ( 
	id	       INTEGER PRIMARY KEY AUTOINCREMENT,
	irt	       VARCHAR(256),
	remarks	       VARCHAR(4096),
	address	       VARCHAR(256),
	mntner         VARCHAR(256),
	fax	       VARCHAR(32),
	email_id       INTEGER,
	signature      VARCHAR(256),
	encryption     VARCHAR(256),
	adminc_id      INTEGER,	   
	techc_id       INTEGER,
	auth	       VARCHAR(256),
	irt_nfy	       INTEGER, -- points to an email
	notify_id      INTEGER,
	mnt_by_id      INTEGER,	
	changed	       INTEGER,
	source_id      INTEGER,	             	
        FOREIGN KEY (source_id) REFERENCES source(id),
	UNIQUE(irt)
);


CREATE TABLE inetinfo2mntirt (
       inetinfo_id    INTEGER,
       mntirt_id      INTEGER,
       FOREIGN KEY (inetinfo_id) REFERENCES inetinfo(id),
       FOREIGN KEY (mntirt_id) REFERENCES irt(id),
       UNIQUE (inetinfo_id, mntirt_id)
);

-- organisation object. Definition at 
-- http://www.ripe.net/db/support/organisation.html

CREATE TABLE organisation ( 
	id	       INTEGER PRIMARY KEY AUTOINCREMENT,
	organisation   VARCHAR(256),
	orgname	       VARCHAR(256),
	descr          VARCHAR(4096),
	remarks        VARCHAR(4096),
	address	       VARCHAR(256),
	phone          VARCHAR(32),
	fax	       VARCHAR(32),
	email_id       INTEGER,
	org	       VARCHAR(256),
	adminc_id      INTEGER,	   
	techc_id       INTEGER,
	ref_nfy	       INTEGER, -- points to an email
	mntref         INTEGER,
	notify_id      INTEGER,
	mnt_by_id      INTEGER,	
	changed	       INTEGER,
	source_id      INTEGER,
        FOREIGN KEY (source_id) REFERENCES source(id),
	UNIQUE(organisation)
);


CREATE TABLE inetinfo2organisation (
       inetinfo_id    INTEGER,
       organisation_id  INTEGER,
       FOREIGN KEY (inetinfo_id)     REFERENCES inetinfo(id),
       FOREIGN KEY (organisation_id) REFERENCES organisation(id),
       UNIQUE (inetinfo_id, organisation_id)
);


-- source table

CREATE TABLE source (
       id     INTEGER PRIMARY KEY AUTOINCREMENT,
       source VARCHAR(32),
       UNIQUE (id, source)
);